@@ -36,7 +36,7 @@ EXAMPLE : use as writer from Databroker with customizations::
 
 """
 
-# Copyright (c) 2017-, UChicago Argonne, LLC.  See LICENSE file.
+# Copyright (c) 2017-2018, UChicago Argonne, LLC.  See LICENSE file.
 
 
 from collections import OrderedDict
@@ -202,9 +202,9 @@ class SpecWriterCallback(object):
             bulk_events = self.bulk_events,
             stop = self.stop,
         )
-        logger = logging.getLogger(__name__)
-        logger.debug("{} document, uid={}".format(key, document["uid"]))
         if key in xref:
+            logger = logging.getLogger(__name__)
+            logger.debug("{} document, uid={}".format(key, document["uid"]))
             self._datetime = datetime.datetime.fromtimestamp(document["time"])
             xref[key](document)
         else:
@@ -269,7 +269,8 @@ class SpecWriterCallback(object):
         self._streams[doc["uid"]] = doc
         
         if doc["name"] == "primary":
-            self.data.update({k: [] for k in sorted(doc["data_keys"].keys())})
+            doc_data_keys = list(doc["data_keys"].keys())
+            self.data.update({k: [] for k in sorted(doc_data_keys)})
             self.data["Epoch"] = []
             self.data["Epoch_float"] = []
         
@@ -291,8 +292,8 @@ class SpecWriterCallback(object):
             if len(self.detectors) > 0:
                 # find 1st detector and move to last
                 det_name = list(self.detectors.keys())[0]
-                if det_name not in self.data:
-                    det_name = list(doc["data_keys"].keys())[0]
+                if det_name not in self.data and len(doc_data_keys) > 0:
+                    det_name = doc_data_keys[0]
                 if det_name in self.data:
                     self.data.move_to_end(det_name)
 
@@ -365,8 +366,21 @@ class SpecWriterCallback(object):
         if len(self.data.keys()) > 0:
             lines.append("#L " + "  ".join(self.data.keys()))
             for i in range(self.num_primary_data):
-                s = [str(self.data[k][i]) for k in self.data.keys()]
+                str_data = OrderedDict()
+                s = []
+                for k in self.data.keys():
+                    datum = self.data[k][i]
+                    if isinstance(datum, str):
+                        # SPEC scan data is expected to be numbers
+                        # this is text, substitute the row number 
+                        # and report after this line in a #U line
+                        str_data[k] = datum
+                        datum = i
+                    s.append(str(datum))
                 lines.append(" ".join(s))
+                for k in str_data.keys():
+                    # report the text data
+                    lines.append("#U {} {} {}".format(i, k, str_data[k]))
         else:
             lines.append("#C no data column labels identified")
 
